; ----------------------------------------------------------------------------
;
; Name, Version & History
; GalaxyGen
; Author	TG Tan		www.angelfire.com\space2\tgtan
; V1.0		4/2/07		1st version
; V1.01		7/2/07		changed default drive
; V1.1		10/2/07		imposed upper limit on variation in Dec, a search intensity factor, an option whether to log the run
; V1.2		18/2/07		bug fix.  Enable the script to generate targets that cross over the 24hr boundary, changed run duration to hrs
; V2.0		11/3/07		added calculation of time residual (actual progress of RA vs elapsed time) and addition of a file to record these residuals
; V2.01		11/3/07		added series no. of target to residuals file
; V2.02		30/3/07		fixed bug - residual not calculated corrected when going past the 24hr RA boundary
; V2.03		9/8/08		Changed label to provide guidance that the download time for 2x2 binning is 9s and for 1x1 is 23s (incr of 3s)
; V2.04		16/8/08		Changed download times to 8s and 22s
; V2.05		24/8/08		Changed download times to 9s and 23s
; 
; Author:         TG Tan
;
; Script Function:
;  Reads a text file and produces 4 files: one as input to TargetPoint, another suitable for automated download of DSS images from CADC, a log file,
;  and a Residuals file. 
;
; Assumptions:
;
; 1.  Input file format (.txt) is Target name, RA, Dec, mag in the form "AAAAA....<tab> HH <tab> MM.mm <tab> DD <tab> MM.mm <tab> MM.mm"
; 2.  Input file is sorted in order of increasing RA.
; ----------------------------------------------------------------------------
Global $ver = "2.05"
;**************************************************************
;  some initial preparations...
;***************************************************************

#include <Date.au3>
#include <file.au3>
#include <GuiConstants.au3>

HotKeySet("^!x", "My_Final_Exit") ; exit the script if Ctrl Alt x is pressed

; The following variables are divided into input, and internal ones 

Global $run_duration	; how long to acquire targets for - in hrs
Global $RA_at_start		; this tells the script the target altitude of the scope through the session - in hrs (HH.hhh)
Global $DEC_reference	; this is the reference declination that that the script will attempt to keep as close as poss to (in DD.ddd)
Global $DEC_max_range	; in deg. no target accepted if more than this away from $DEC_reference
Global $Search_Intensity	; this factor reduces the RA search window and increases the number of potential targets 
Global $num_exp			; the number of exposures per target
Global $exp_time		; in s
Global $download_time	; in s
Global $scope_settle_time	; in s
Global $path			; location of the input and output files

Global $time_elapsed	; estimated duration into the run at the current target in hrs - used to calculate time residuals
Global $time_residual
Global $reference_RA	; The absolute RA at the start of the session - used to calculate time residuals - in
Global $RA_delta			; difference between current RA and reference_RA
;Global $max_time_residual = 0.25	; in hrs
Global $catalog_file
Global $targetpoint_file
Global $cadc_file
Global $log_file
Global $residuals_file
Global $targets_total		; the number of targets identified
Global $RA_shift
Global $start_line
Global $end_line		; these are the lines nos of the catalog file bounding the range of galaxies within the DEC range
Global $targets_done
Global $total_lines
Global $date			; this is the current date stripped of forward slashes
Global $log_on = $GUI_UNCHECKED		; if checked, write actions to log file
Global $RA_end_buffer = 1	; the additional RA value (in hrs) to add to $run_duration when calculating when to end target ID


;*************************************************************************
; initialise the start values of the other variables, to be used on initial run, or if the params file doesn't exist
;*************************************************************************
If FileExists("param_galaxygen.txt") Then
	$params_file = FileOpen("param_galaxygen.txt", 0)
	
	$run_duration = FileReadLine($params_file)
	$RA_at_start = FileReadLine($params_file)
	$DEC_reference = FileReadLine($params_file)
	$DEC_max_range = FileReadLine($params_file)
	$Search_Intensity = FileReadLine($params_file)
	$num_exp = FileReadLine($params_file)
	$exp_time = FileReadLine($params_file)
	$download_time = FileReadLine($params_file)
	$scope_settle_time = FileReadLine($params_file)
	$path = FileReadLine($params_file)			
	
	FileClose($params_file)
	
Else; on an initial run, use these defaults
	
	$run_duration = 3
	$RA_at_start = 0
	$DEC_reference = -35
	$DEC_max_range = 12
	$Search_Intensity = 1
	$num_exp = 15
	$exp_time = 20
	$download_time = 9		; est. 9s for 2x2 binning, 23s for 1x1 
	$scope_settle_time = 20
	$path = "H:\AstroData\SN_Search\"
		
EndIf

;*****************************************************************************
; initialise some variables
;*****************************************************************************

$targets_total = 0		; a temporary number
$date = StringReplace(_NowDate(), "/", "")


;*****************************************************************************
; This block contains code for the script graphical interface
;******************************************************************************


opt("GuiOnEventMode", 1) ; change to On Event mode
$main_window = GUICreate("GalaxyGen, V" & $ver & " by TG Tan", 360, 500, 0, 0)

; input fields

GUICtrlCreateLabel("Run duration (hrs):", 10, 21)
$run_duration_box = GUICtrlCreateInput($run_duration, 270, 16, 50, 20)

GUICtrlCreateLabel("RA of first target (Hrs, HH.hh):", 10, 49)
$RA_at_start_box = GUICtrlCreateInput($RA_at_start, 270, 44, 50, 20)

GUICtrlCreateLabel("Ref Declination of targets (degs):", 10, 77)
$DEC_reference_box = GUICtrlCreateInput($DEC_reference, 270, 72, 50, 20)

GUICtrlCreateLabel("Max Declination deviation tolerated (degs):", 10, 105)
$DEC_max_range_box = GUICtrlCreateInput($DEC_max_range, 270, 100, 50, 20)

GUICtrlCreateLabel("Search Intensity Factor ( try between 1 and 1.5 ):", 10, 133)
$Search_Intensity_box = GUICtrlCreateInput($Search_Intensity, 270, 128, 50, 20)

GUICtrlCreateLabel("Number of exposures planned per target:", 10, 161)
$num_exp_box = GUICtrlCreateInput($num_exp, 270, 156, 50, 20, $ES_NUMBER)

GUICtrlCreateLabel("Exp time planned per target (s):", 10, 189)
$exp_time_box = GUICtrlCreateInput($exp_time, 270, 184, 50, 20, $ES_NUMBER)

GUICtrlCreateLabel("Download time - 9s for 2x2 and 23s for 1x1 (s):", 10, 217)
$download_time_box = GUICtrlCreateInput($download_time, 270, 212, 50, 20, $ES_NUMBER)

GUICtrlCreateLabel("Scope settle time after slew (s):", 10, 245)
$scope_settle_time_box = GUICtrlCreateInput($scope_settle_time, 270, 240, 50, 20, $ES_NUMBER)

GUICtrlCreateLabel("Input and output file path:", 10, 280)
$path_box = GUICtrlCreateInput($path, 170, 275, 160, 20, $ES_AUTOHSCROLL)

; make a Browse button for the path

$browse_button = GUICtrlCreateButton("Browse", 270, 300, 50, 30)
GUICtrlSetOnEvent($browse_button, "Browse_path") ; when button is pressed, locate the desired path

$log_checkbox = GUICtrlCreateCheckbox(" Log current run?", 30, 310)

;*********************************************************************************
; the status display part of the GUI
;*********************************************************************************

; make a display that keeps track of the no. of targets done

$targets_done_label = GUICtrlCreateLabel("Targets done:", 20, 422, 160, 50)
GUICtrlSetFont($targets_done_label, 12, 600)
$targets_done_box = GUICtrlCreateInput($targets_done, 150, 415, 70, 32, $ES_READONLY)
GUICtrlSetFont($targets_done_box, 20, 800)

$targets_total_label = GUICtrlCreateLabel("of target", 230, 422, 160, 50)
GUICtrlSetFont($targets_total_label, 12, 600)
$targets_total_number = GUICtrlCreateLabel($targets_total, 310, 417, 160, 50)
GUICtrlSetFont($targets_total_number, 20, 800)

; housekeeping functions for the main window

$ok_button = GUICtrlCreateButton("RUN", 100, 345, 70, 40)
GUICtrlSetOnEvent($ok_button, "Execute_Main") ; when button is pressed, proceed
$stop_button = GUICtrlCreateButton("CANCEL", 200, 345, 70, 40)
GUICtrlSetOnEvent($stop_button, "My_Final_Exit") ; ends the script

$exit_label = GUICtrlCreateLabel("Once RUN activated, Ctrl-Alt-X to Exit.", 10, 460, 380, 110)
GUICtrlSetFont($exit_label, 9, 800, 2)

GUISetOnEvent($GUI_EVENT_CLOSE, "My_Final_Exit") ; pressing X ends the script
GUISetOnEvent($GUI_EVENT_MINIMIZE, "Minimize")
GUISetOnEvent($GUI_EVENT_RESTORE, "Restore")

GUISetState(@SW_SHOW)
Wait_For_Input()

;-------------------------------------------------------------------------------------------------------------
;
;  End of main program block
;
;-------------------------------------------------------------------------------------------------------------

;******************************************************************
; this function is a loop that keeps the script going while data is being input
;******************************************************************

Func Wait_For_Input()
	
	While 1
		Sleep(500)		; this delay tries to avoid excessive processor churning
	WEnd
EndFunc   ;==>Wait_For_Input


;************************************************************************************
;
;  this contains the main loop of the program
;
;***********************************************************************************

Func Execute_Main()
	
	; the Run and Cancel buttons stop working here
	
	GUICtrlSetState($stop_button, $GUI_DISABLE)
	GUICtrlSetState($ok_button, $GUI_DISABLE)
	
	; assign parameters
	
	$run_duration = GUICtrlRead($run_duration_box)
	$RA_at_start = GUICtrlRead($RA_at_start_box)
	$DEC_reference = GUICtrlRead($DEC_reference_box)
	$DEC_max_range = GUICtrlRead($DEC_max_range_box)
	$Search_Intensity = GUICtrlRead($Search_Intensity_box)
	$num_exp = GUICtrlRead($num_exp_box)
	$exp_time = GUICtrlRead($exp_time_box)
	$download_time = GUICtrlRead($download_time_box)
	$scope_settle_time = GUICtrlRead($scope_settle_time_box)
	$path = GUICtrlRead($path_box)
	$log_on = GUICtrlRead($log_checkbox)
	
	If StringRight($path, 1) <> "\" Then $path = $path & "\"		; make sure the path parameter has the correct form 

	;*************************************************************
	; write the parameters to file so they can be re-used
	;*************************************************************
	
	$params_file = FileOpen("param_galaxygen.txt", 2)
	
	FileWriteLine($params_file, $run_duration)
	FileWriteLine($params_file, $RA_at_start)
	FileWriteLine($params_file, $DEC_reference)
	FileWriteLine($params_file, $DEC_max_range)
	FileWriteLine($params_file, $Search_Intensity)
	FileWriteLine($params_file, $num_exp)
	FileWriteLine($params_file, $exp_time)
	FileWriteLine($params_file, $download_time)
	FileWriteLine($params_file, $scope_settle_time)
	FileWriteLine($params_file, $path)
	
	FileClose($params_file)


	;************************************************************
	; calculations
	;************************************************************
	
	$RA_shift = ($num_exp * ($exp_time + $download_time) + $scope_settle_time)/60/60/$Search_Intensity		; in hrs, the Search Intensity Factor increases the no. of targets returned
	$targets_total = Round($run_duration / $RA_shift / $Search_Intensity)
	
	GUICtrlSetData($targets_total_number, $targets_total)
	
	;*************************************************************************
	; Open the target list for reading, if it's not there, exit
	;*************************************************************************
	If FileExists($path & "catalog.txt") Then
		$total_lines = _FileCountLines($path & "catalog.txt")
		$catalog_file = FileOpen($path & "catalog.txt", 0)
		
	Else
		MsgBox(0, "Error", "The file " & $path & "catalog.txt not found.")
		Exit
	EndIf

	;************************************************************
	; open files for writing
	;************************************************************
	
	$targetpoint_file = FileOpen($path & "targetpoint" & $date & ".txt", 2)
	$cadc_file = FileOpen($path & "cadc" & $date & ".txt", 2)
	$residuals_file = FileOpen($path & "Residuals.txt", 2)
	
	
	If $log_on = $GUI_CHECKED Then
		$log_file = FileOpen($path & "galaxygen_log.txt", 1)			; append to end of existing log file
		
		;*****************************************************************
		; make log file header for this run
		;*****************************************************************
		
		FileWriteLine($log_file, "*************************************************************************************")
		FileWriteLine($log_file, "GalaxyGen " & "ver" & $ver & " Run started: " & _Now())
		FileWriteLine($log_file, "Parameters: ")
		FileWriteLine($log_file, "Run Duration (hrs): " & $run_duration)
		FileWriteLine($log_file, "RA at start (hrs): " & $RA_at_start)
		FileWriteLine($log_file, "DEC reference (deg): " & $DEC_reference)
		FileWriteLine($log_file, "DEC max dev (deg): " & $DEC_max_range)
		FileWriteLine($log_file, "Search Intensity Factor: " & $Search_Intensity)
		FileWriteLine($log_file, "No. of exposures: " & $num_exp)
		FileWriteLine($log_file, "Exp time (s): " & $exp_time)
		FileWriteLine($log_file, "Download time (s): " & $download_time)
		FileWriteLine($log_file, "Scope settle time (s): " & $scope_settle_time)
		FileWriteLine($log_file, "")
		FileWriteLine($log_file, "Est time between targets (min): " & $RA_shift*60* $Search_Intensity)
		FileWriteLine($log_file, "")
		FileWriteLine($log_file, "Targets identified:")
	EndIf
	
	;*****************************************************************
	; identify targets
	;*****************************************************************
	Local $input_line[7] ; this receives the parsed input line from the catalog file
	Local $cur_line_no
	Local $cur_target_line
	Local $cur_line_RA
	Local $cur_min_RA
	Local $cur_max_RA
;	Local $RA_delta			; difference between current RA and reference_RA
	Local $cur_line_DEC
	Local $best_target_DEC
	Local $best_target[7]		; holds the target that is closest so far to the mid value of the Dec range
	Local $not_yet_in_range		; RA of current line is not yet in range
	Local $in_range
	Local $just_passed_range
	Local $wait_time
	
	$targets_done = 0
	$reference_RA = $RA_at_start
	$time_elapsed = 0
	
	
	$best_target[4] = 0		; the zero setting is used to mark no target found so far, ie relies on the fact that we have no catalog entries at DEC = 0
	$best_target[5] = 0		; initialise the best matched DEC so far
	
	$theoretical_RA_end = $RA_at_start + $run_duration + $RA_end_buffer
	If $theoretical_RA_end > 24 Then
		$RA_end = 24
	Else
		$RA_end = $theoretical_RA_end
	EndIf
	
	$next_pass_RA_end = $theoretical_RA_end - 24		; this will be negative if a second pass through the catalog is not needed
	$pass_needed = 1
	
	While $pass_needed			; do at least 1 pass
	
	;-----------------------------------------------------------------------
	; code within may be run twice if the RA range straddles the 24 hr boundary 
	
	

		$cur_min_RA = $RA_at_start
		$cur_max_RA = $RA_at_start + $RA_shift
	
		For $cur_line_no = 1 To $total_lines		; as long as we haven't got to the end of the catalog file
			$cur_target_line = FileReadLine($catalog_file)
	
			$input_line = StringSplit($cur_target_line, Chr(9))		; the file is tab-delimited
		
			$cur_line_RA = Number($input_line[2]) + Number($input_line[3])/60
			$cur_line_DEC = Number($input_line[4]) - Number($input_line[5])/60		; assume all DEC are negative
		
			$best_target_DEC = $best_target[4] - $best_target[5]/60
		
			If $cur_line_RA < $cur_min_RA Then
				$not_yet_in_range = 1
				$in_range = 0
				$just_passed_range = 0
			ElseIf $cur_line_RA < $cur_max_RA Then
				$not_yet_in_range = 0
				$in_range = 1
				$just_passed_range = 0
			Else
				$not_yet_in_range = 1
				$in_range = 0
				$just_passed_range = 1
			EndIf
					
			If $in_range Then		; current line is within RA range
				if Abs($cur_line_DEC - $DEC_reference) < Abs($best_target_DEC - $DEC_reference) Then
					$best_target = $input_line
				EndIf
			EndIf
		
			If $just_passed_range Then		
				If $best_target[4] <> 0 And Abs($best_target_DEC - $DEC_reference) < $DEC_max_range And $cur_line_RA < $RA_end Then 
					; do not accept DEC that are too far from ref, or targets that are beyond $RA_end
					;***********************************************************
					; work out the time residual
					;***********************************************************
					
					$time_elapsed = $time_elapsed + ($num_exp * ($exp_time + $download_time) + $scope_settle_time)/60/60	; in hrs
					$RA_delta = $cur_line_RA - $reference_RA
					$time_residual = $RA_delta - $time_elapsed
					If $cur_line_RA < $reference_RA Then $time_residual = $time_residual + 24	; covers cases where the 24 hr RA boundary is crossed
					;If $time_residual > $max_time_residual Then
					;	$wait_time = $RA_delta - $time_elapsed
					;	$time_elapsed = $time_elapsed + $wait_time
					;Else
					;	$wait_time = 0
					;EndIf
					
					;***********************************************************
					
					$targets_done = $targets_done + 1					
					Write_Output_Files($best_target)	; write to file if we have found a suitable target

					GUICtrlSetData($targets_done_box, $targets_done)
					;If $targets_done = $targets_total Then ExitLoop				; finish when we have done all targets
				EndIf
			
				$cur_min_RA = $cur_min_RA + $RA_shift
				$cur_max_RA = $cur_max_RA + $RA_shift	; reset the RA window
				$best_target[4] = 0
				$best_target[5] = 0		; reset the best matched DEC so far
			
			EndIf
				
		Next
	
		If $next_pass_RA_end > 0 Then				; another pass is needed
			$RA_at_start = 0
			$RA_end = $next_pass_RA_end
			FileClose($catalog_file)
			$catalog_file = FileOpen($path & "catalog.txt", 0)		; resets the catalog file for reading again
			$next_pass_RA_end = -1									; set this so that the script exits after the 2nd pass
		Else
			ExitLoop
		EndIf
	
	
	; code within may be run twice if the RA range straddles the 24 hr boundary 
	;------------------------------------------------------------------------------------------------------------------
	WEnd
	
	FileClose ($targetpoint_file)
	FileClose ($cadc_file)
	FileClose ($catalog_file)
	FileClose ($residuals_file)
	
	; complete the log file
	If $log_on = $GUI_CHECKED Then
		FileWriteLine($log_file, "--------------------------------")	
		FileWriteLine($log_file, "Run ended: " & _Now())	
		FileWriteLine($log_file, "*************************************************************************************")
		FileClose ($log_file)
	EndIf
	
	; the Cancel button start working again
	
	GUICtrlSetState($stop_button, $GUI_ENABLE)
	GUICtrlSetState($ok_button, $GUI_ENABLE)
		
EndFunc   ;==>Execute_Main

;*****************************************************************
; Write the selected target details to the appropriate files
;*****************************************************************

Func Write_Output_Files($target)
	FileWriteLine($targetpoint_file, $target[1] & " " & $target[2] & " " & $target[3] & " " & $target[4] & " " & $target[5])
	FileWriteLine($cadc_file, $target[1] & Chr(9) & $target[1] & Chr(9) & "10")	; the format is Label <tab> Object name <tab> Size of image in mins
	FileWriteLine($log_file, $target[1] & " " & $target[2] & " " & $target[3] & " " & $target[4] & " " & $target[5] & " " & $target[6])
	FileWriteLine($residuals_file, String($targets_done) & " " & $target[1] & " " & $target[2] & " " & $target[3] & " " & $target[4] & " " & $target[5] & " Elapsed " & String(Round($time_elapsed,1)) & " Resid " & String(Round($time_residual, 1)))
EndFunc		;==>Write_Output_Files


;***********************************************
; window housekeeping functions
;***********************************************

Func Minimize()
	GUISetState(@SW_MINIMIZE)
EndFunc   ;==>Minimize

Func Restore()
	GUISetState(@SW_RESTORE)
EndFunc   ;==>Restore

Func Close_Window()
	GUIDelete()
EndFunc   ;==>Close_Window


;****************************************************************************
; Browse for the file save path and update the path input field when done
;***************************************************************************

Func Browse_path()
	$newpath = FileSelectFolder("Select save path", "", 1, $path)
	If $newpath <> "" Then $path = $newpath
	GUICtrlSetData($path_box, $path)
EndFunc


;***********************************************
; exit
;************************************************
 
Func My_Final_Exit()

	Exit; this completely terminates the script and closes the interface window
EndFunc   ;==>My_Final_Exit

;**************************************************************************************************
;  END of the script.
;**************************************************************************************************